/********************** (C)Kirk Robinson Controls *****************************
*
* File Name          :  PCA9555.c
* Description        :  application for I2C PCF9555 IO expander
*			using software I2C on CC2540 USB dongle
*			(wires attached to P1.4 & P1.5 pads)
*			
*******************************************************************************/
//    _-_-_-_-_-_-_	STPrime	October 28, 2011	_-_-_-_-_-_-_

#include "hal_types.h"
#include "PCA9555.h"
#include "hal_i2c.h"
#include "LCD.h"

/* Public variables ----------------------------------------------------------*/
uint8 buffer[32];									// buffer for DRAW formating
uint8 data1[4];									// array for ADC/DAC values
uint32 TimeOut;									// I2C test EV* timer
uint8 ValueTextBuffer[5];		// float value to string
uint8	LCD_Refresh_Screen;		// how often LCD is updated
uint8 PCA9555_Present = 1;		// test for device
uint8 PCA9555_ConfigP0;			// Configuration for P0
uint8 PCA9555_ConfigP1;			// Configuration for P1
uint8 PCA9555_OutPort0;			// Output for P0
uint8 PCA9555_OutPort1;			// Output for P1
uint8 PCA9555_InPort0;			// Read pins for P0
uint8 PCA9555_InPort1;			// Read pins for P1
uint8 PCA9555_PolP0;			// Polarity of Port0 Pins
uint8 PCA9555_PolP1;			// Polarity of Port1 Pins


/* Private function prototypes -----------------------------------------------*/
static void _int2str( uint8* ptr, int X, int digit, bool flagunsigned, bool fillwithzero );

/* Private functions ---------------------------------------------------------*/

/*******************************************************************************
*
*                    _int2str
*
*******************************************************************************/
/**
*
*  Translate a integer into a string.
*
*  @param[in,out] ptr            A pointer to a string large enough to contain
*                                the translated 32 bit word.
*  @param[in]     X              The integer to translate.
*  @param[in]     digit          The amount of digits wanted in the result string.
*  @param[in]     flagunsigned   Is the input word unsigned?
*  @param[in]     fillwithzero   Fill with zeros or spaces.
*
**/
/******************************************************************************/
static void _int2str( uint8* ptr, int X, int digit, bool flagunsigned, bool fillwithzero )
{
    uint8      c;
    bool    fFirst   = 0;
    bool    fNeg     = 0;
    uint8  DIG      = 1;
    int   i;
    int8   r;

    for ( i = 1; i < digit; i++ )
    {
        DIG *= 10;
    }

    if ( !flagunsigned && ( X < 0 ) )
    {
        fNeg = 1;
        r    = -X;
    }
    else
    {
        r = X;
    }

    for ( i = 0; i < digit; i++, DIG /= 10 )
    {
        c  = ( r / DIG );
        r -= ( c * DIG );

        if ( fillwithzero || fFirst || c || ( i == ( digit - 1 ) ) )
        {
            if (( fFirst == 0 ) && !flagunsigned )
            {
                *ptr++ = fNeg ? '-' : ' ';
            }

            *ptr++ = ( c % 10 ) + '0';
            fFirst = 1;
        }
        else
        {
            *ptr++ = ' ';
        }
    }

    *ptr++ = '\0';
}


/* Public functions ----------------------------------------------------------*/


/*******************************************************************************
*
*                   UTIL_uint2str
*
*******************************************************************************/
/**
*
*  Convert an <b>unsigned</b> integer into a string.
*  Using this function leads to much smaller code than using the sprintf()
*  function from the C library.
*
*  @param [out]  ptr    The output string.
*  @param [in]   X      The unsigned value to convert.
*  @param [in]   digit  The number of digits in the output string.
*  @param [in]   fillwithzero  \li 0   fill with blanks.
*                              \li 1   fill with zeros.
*
**/
/********************************************************************************/
void UTIL_uint2str( uint8* ptr, uint8 X, uint8 digit, bool fillwithzero )
{
    _int2str( ptr, X, digit, 1, fillwithzero );
}

/*******************************************************************************
*
*                   UTIL_int2str
*
*******************************************************************************/
/**
*
*  Convert a <b>signed</b> integer into a string.
*  Using this function leads to much smaller code than using the sprintf()
*  function from the C library.
*
*  @param [out]  ptr    The output string.
*  @param [in]   X      The unsigned value to convert.
*  @param [in]   digit  The number of digits in the output string.
*  @param [in]   fillwithzero  \li 0   fill with blanks.
*                              \li 1   fill with zeros.
*
**/
/******************************************************************************/
void UTIL_int2str( uint8* ptr, int X, uint8 digit, bool fillwithzero )
{
    _int2str( ptr, X, digit, 0, fillwithzero );
}

/**
  * @brief  Example of timeout situation management.
  * @param  None.
  * @retval None.
  */
uint8 I2C2_TIMEOUT_UserCallback(void)
{
  /* Use application may try to recover the communication by resetting I2C
    peripheral (calling the function I2C_SoftwareResetCmd()) then re-start
    the transmission/reception from a previously stored recover point.
    For simplicity reasons, this example only shows a basic way for errors
    managements which consists of stopping all the process and requiring system
    reset. */


  /* Display error message on screen */
 //   sprintf(msg_buf, "! ERROR !");

	return I2C_ERROR;
}
/*******************************************************************************
*
*                                UTIL_int2float2str
*
*******************************************************************************/
/**
*
*  This function is used to display the current ADC/DAC values with.
*  two decimal places
*
*
**/
/******************************************************************************/
void UTIL_int2float2str( uint32 value )
{
    uint8 c;
    uint16 r = 0;

    // Display modified ADC/DAC value by dividing the 3 digit value by a
    // factor of 10 each time and then inserting the decimal point
    /* 1 000 digit*/
    c = (( value - r ) / 1000 );
    r = r + ( c * 1000 );
    // if the leading value is zero then display a blank space
	if (c){
		ValueTextBuffer[0] = c + 0x30;
	}else{
		ValueTextBuffer[0] = 0x20;		
	}
    /* 100 digit*/
    c = (( value - r ) / 100 );
    r = r + ( c * 100 );
    ValueTextBuffer[1] = c + 0x30;

    /* Dot*/
    ValueTextBuffer[2] = '.';

    /* 10 digit*/
    c = (( value - r )/10);
    r = r + ( c * 10 );
    ValueTextBuffer[3] = c + 0x30;

    /* 1 digit*/
    c = (( value - r ));
    r = r +  c ;
    ValueTextBuffer[4] = c + 0x30;

    /* Volt*/
 //   ValueTextBuffer[5] = 'V';
}

/*******************************************************************************
*
*                                UTIL_integer2str
*
*******************************************************************************/
/**
*
*  This function is used to display the current ADC/DAC values with.
*  two decimal places
*
*
**/
/******************************************************************************/
void UTIL_integer2str( uint16 value )
{
    uint8 c;
    uint16 r = 0;

    // Display modified ADC/DAC value by dividing the 3 digit value by a
    // factor of 10 each time and then inserting the decimal point
    /* 1 000 digit*/
    c = (( value - r ) / 1000 );
    r = r + ( c * 1000 );
    // if the leading value is zero then display a blank space
	if (c){
		ValueTextBuffer[0] = c + 0x30;
	}else{
		ValueTextBuffer[0] = 0x20;		
	}
    /* 100 digit*/
    c = (( value - r ) / 100 );
    r = r + ( c * 100 );
    ValueTextBuffer[1] = c + 0x30;

    /* 10 digit*/
    c = (( value - r )/10);
    r = r + ( c * 10 );
    ValueTextBuffer[2] = c + 0x30;

    /* 1 digit*/
    c = (( value - r ));
    r = r +  c ;
    ValueTextBuffer[3] = c + 0x30;

}



/* Initialization of PCA9555 I/O Expander.  Called whenever a change is made to the
	configuration or polarity register to update LCD */
void PCA9555_INIT(void)
{
  // Reset I2C controller
  HalI2CInit();
  // Initialize PCA9555 I/O Expander Ports
  // Set initial configuration values here so that the PCA9555_INIT can be
  // called again when changes are made to the original configuration

  // all Port0 Pins are outputs
  PCA9555_ConfigP0 = PCA9555_Pin_CLEAR;		
  // all Port1 Pins are outputs
  PCA9555_ConfigP1 = PCA9555_Pin_CLEAR;
  // all Port0 Pins are Low
  PCA9555_OutPort0 = 0x00;
  // all Port1 Pins are Low
  PCA9555_OutPort1 = 0x00;
  // Polarity of the pins are set to zero Low = 0, High = 1
  PCA9555_PolP0 = PCA9555_Pin_CLEAR;
  PCA9555_PolP1 = PCA9555_Pin_CLEAR;
	
  // Test if PCA9555 device is present
  if ( I2C_ReadMultByte(PCA9555_ADDR_WR, InPort0, data1, 2 )  )
  {
    // Test for input from Port 0
    // read from ValueTextBuffer
    UTIL_integer2str( data1[0] );
    // Test for input from Port 1
    UTIL_integer2str( data1[1] );

    data1[0] = PCA9555_ConfigP0;
    data1[1] = PCA9555_ConfigP1;
    I2C_WriteMultByte(PCA9555_ADDR_WR, ConfigP0, data1, 2);
	
    // Invert polarity of ports so that a Low is 0 and High is 1 for both Ports
    data1[0] = PCA9555_PolP0;
    data1[1] = PCA9555_PolP1;
    I2C_WriteMultByte(PCA9555_ADDR_WR, PolInvP0, data1, 2);

    // Set up the PCA9555 IO Expander Port0 for ConfigP0 I/O
    data1[0] = PCA9555_OutPort0;
    data1[1] = PCA9555_OutPort1;
    I2C_WriteMultByte(PCA9555_ADDR_WR, OutPort0, data1, 2);
	
    // Read registers of the PCA9555 and display values
    I2C_ReadMultByte(PCA9555_ADDR_WR, ConfigP0, data1, 1);
    // Verify that the ports are CONFIGURED properly
    // read from ValueTextBuffer
    UTIL_integer2str( data1[0] );
									
    // Verify that the INPUT PORTS are configured as inputs
    I2C_ReadMultByte(PCA9555_ADDR_WR, InPort0, data1, 1);
    // read from ValueTextBuffer
    UTIL_integer2str( data1[0] );
									
    // Verify that the OUTPUT PORTS are configured as outputs

    I2C_ReadMultByte(PCA9555_ADDR_WR, OutPort0, data1, 2);
    // read from ValueTextBuffer
    UTIL_integer2str( data1[0] );
									
    // Verify that the port POLARITY are configured
    I2C_ReadMultByte(PCA9555_ADDR_WR, PolInvP0, data1,2 );
    // read from ValueTextBuffer
    UTIL_integer2str( data1[0] );

    // Successful Initialization of PCA9555		
    PCA9555_Present = 1;
  }else{
    // FAIlED initialization of PCA9555
    PCA9555_Present = 0;
  }
	
  // Read Port0 and Port1 values, returns an error message if fails
  // The Port address is auto incremented,
  // and the values are stored in the data1[] array
  if ( I2C_ReadMultByte(PCA9555_ADDR_WR, InPort0, data1, 2) )
  {	
  // LCD_Refresh only updates screen after set time period
  //while (!LCD_Refresh_Screen--)
  //{
  // Read Port 0 value and format data from integer to string for LCD
    UTIL_integer2str( data1[0] );
  // Read Port 1 value and format data from integer to string for LCD
    UTIL_integer2str( data1[1] );
	
   }else{
  // call function to handle error cause by timeout of I2C read
  }
}


/*******************************************************************************
*
*                            PCA9555_IOPinConfig
*
*******************************************************************************/
/**
  * @brief  Configures the selected pin direction (to be an input or an output)
  * @param  DeviceAddr: The address of the IOExpander, could be : PCA9555_ADDR_WR.
  * @param  IO_Pin: IO_Pin_x: Where x can be from 0 to 7.
  * @param  Port: x_P_y: Where x InPort, OutPort, ConfigP, PolInv y can be from 0 or 1.
  * @param  Direction: could be Direction_IN or Direction_OUT.
  * @retval PCA9555_OK: if all initializations are OK. Other value if error.
  */
uint8 PCA9555_IOPinConfig(uint8 DeviceAddr, uint8 Port,
			    uint8 IO_Pin, uint8 Direction)
{

  /* Get the current Pin direction (Pin value) of port */
  I2C_ReadMultByte(PCA9555_ADDR_WR, Port, data1, 1);
  if (Direction != PCA9555_Direction_IN)
  {
    data1[0] |= (uint8)IO_Pin;
  }
  else
  {
    data1[0] &= ~(uint8)IO_Pin;
  }

  /* Write the register new value */
//	HalI2CSend(PCA9555_ADDR_WR, data1[0], Port);

  /* If all OK return IOE_OK */
  return I2C_OK;
}

