/********************** (C)Kirk Robinson Controls *****************************
*
* File Name          :  PCA9555.h
* Description        :  application for I2C PCF9555 IO expander
*			using software I2C on CC2540 USB dongle
*			(wires attached to P1.4 & P1.5 pads)
*			
*******************************************************************************/
// _-_-_-_-_-_-_		STPrime			October 28, 2011	_-_-_-_-_-_-_

#include "hal_types.h"

/* Private defines -----------------------------------------------------------*/
#define ADDR_WR   		0x4E    /* addr for WRITE access*/
#define ADDR_RD   		0x4F    /* addr for READ access*/
#define PCA9555_ADDR_WR	        0x4E	/* addr for WRITE access*/
#define PCA9555_ADDR_RD         0x4F	/* addr for READ access*/

// PCA9555 Control Byte values see datasheet PCA9555 16 bit I/O Expander
// SCPS131EAUGUST 2005REVISED MAY 2008 (TI sample)
/*
B2 	B1 	B0
0 	0 	0 	0x00	Input Port 0 			Read byte xxxx xxxx
0 	0 	1 	0x01	Input Port 1 			Read byte xxxx xxxx
0 	1 	0 	0x02	Output Port 0 			R/Wr byte 1111 1111
0 	1 	1 	0x03	Output Port 1 			R/Wr byte 1111 1111
1 	0 	0 	0x04	Polarity Inversion Port 0 	R/Wr byte 0000 0000
1 	0 	1 	0x05	Polarity Inversion Port 1 	R/Wr byte 0000 0000
1 	1 	0 	0x06	Configuration Port 0 		R/Wr byte 1111 1111
1 	1 	1 	0x07	Configuration Port 1 		R/Wr byte 1111 1111
 		   	 ____________
 		   	|	     |  Port 0	
 		   	|            |------- I/0 0
 		   	|I/O Expander|	    - I/0 1
     	   	        | PCA95555   |	    - I/0 2         16 bit I/O expander : ADDR = 0x4E
 		   	| ADDR: 0x27 |------- I/0 3         Two 8 bit Ports
 		   	|	     |      - I/0 4         With Interrupt
 		   	|     	     |	    - I/0 5
                        | 	     |      - I/0 6
 			|	     |------- I/0 7
 			|	     |  Port 1
 			|	     |------- I/0 10
 			|  	     |      - I/0 11	
     		        |            |	    - I/0 12
 			|	     |------- I/0 13
 			|	     |      - I/0 14
 			|  	     |	    - I/0 15
     		        | 	     |	    - I/0 16
     		        | 	     |------- I/0 17
     		        | 	     |		
			|____________|-------- Interrupt Output
*/

#define InPort0		0x00	// read from Port 0
#define InPort1		0x01	// read from Port 1
#define OutPort0	0x02	// read/write Port 0
#define OutPort1	0x03	// read/write Port 1
#define PolInvP0	0x04	// invert value from Port 0
#define PolInvP1	0x05	// invert value from Port 1
#define ConfigP0	0x06	// configure Port 0
#define ConfigP1	0x07	// configure Port 1

#define I2C_SPEED 100000

// Error Flags
#define I2C_ERROR 	0		// error from I2C reads
#define I2C_OK    	1		// I2C read was successful

// Update rate of LCD displaying all the ADC and DAC values
#define UPDATE_RATE 			50

//  * @brief  PCA9555 Expander Interrupt line on EXTI

/*
#define PCA9555_IT_PIN                       GPIO_Pin_12
#define PCA9555_IT_GPIO_PORT                 GPIOA
#define PCA9555_IT_GPIO_CLK                  RCC_APB2Periph_GPIOA
#define PCA9555_IT_EXTI_PORT_SOURCE          GPIO_PortSourceGPIOA
#define PCA9555_IT_EXTI_PIN_SOURCE           GPIO_PinSource12
#define PCA9555_IT_EXTI_LINE                 EXTI_Line12
#define PCA9555_IT_EXTI_IRQn                 EXTI15_10_IRQn
*/



/**
  * @brief IO Expander Pins definition
  */
#define PCA9555_IN_ALL_PINS          (uint16_t)(PCA9555_Pin_ALL)
#define PCA9555_OUT_ALL_PINS         (uint16_t)(PCA9555_Pin_ALL)
/**
  * @brief  IO Pins
  */
#define PCA9555_Pin_0               0x01
#define PCA9555_Pin_1               0x02
#define PCA9555_Pin_2               0x04
#define PCA9555_Pin_3               0x08
#define PCA9555_Pin_4               0x10
#define PCA9555_Pin_5               0x20
#define PCA9555_Pin_6               0x40
#define PCA9555_Pin_7               0x80
#define PCA9555_Pin_ALL             0xFF  // configured as inputs
#define PCA9555_Pin_CLEAR	    0x00  // configured as outputs

#define Port0   0x00
#define Port1   0x01

#define p0_0    0x01
#define p0_1    0x02
#define p0_2    0x04
#define p0_3    0x08
#define p0_4    0x10
#define p0_5    0x20
#define p0_6    0x40
#define p0_7    0x80

#define p1_0    0x01
#define p1_1    0x02
#define p1_2    0x04
#define p1_3    0x08
#define p1_4    0x10
#define p1_5    0x20
#define p1_6    0x40
#define p1_7    0x80

/**
  * @brief  IO Pin directions
  */
#define PCA9555_Direction_IN	    0x00
#define PCA9555_Direction_OUT	    0x01

/**
  * @brief  Interrupt Line output parameters
  */
#define PCA9555_Polarity_Normal	    0x00
#define PCA9555_Polarity_Inverted   0x01


/* Maximum Timeout values for flags and events waiting loops. These timeouts are
   not based on accurate values, they just guarantee that the application will
   not remain stuck if the I2C communication is corrupted.
   You may modify these timeout values depending on CPU frequency and application
   conditions (interrupts routines ...). */
#define I2C_LONG_TIMEOUT         0x5000//((uint32_t)(1 * I2C_FLAG_TIMEOUT))


/* Private functions ---------------------------------------------------------*/

uint8 I2C2_TIMEOUT_UserCallback(void);
void UTIL_int2float2str( uint32 value );
void UTIL_integer2str( uint16 value );
void PCA9555_INIT(void);
uint8 PCA9555_IOPinConfig(uint8 DeviceAddr, uint8 Port,
			    uint8 IO_Pin, uint8 Direction);
extern uint8 ValueTextBuffer[5];		// float value to string
